package data_manage

import (
	"errors"
	"eta/eta_mobile/utils"
	"fmt"
	"github.com/beego/beego/v2/client/orm"
	"github.com/shopspring/decimal"
	"strconv"
	"strings"
	"time"
)

// 时间移位
func EditCalculateTimeShift(req *EdbInfoCalculateBatchEditReq, fromEdbInfo *EdbInfo, edbCode string, oldEdbInfo *EdbInfo) (edbInfoId int, err error) {
	edbInfoId = req.EdbInfoId
	o := orm.NewOrmUsingDB("data")
	to, err := o.Begin()
	if err != nil {
		return
	}
	defer func() {
		if err != nil {
			_ = to.Rollback()
		} else {
			_ = to.Commit()
		}
	}()

	//修改指标信息
	sql := ` UPDATE  edb_info
			SET
			  edb_name =?,
			  edb_name_source=?,
			  frequency = ?,
			  unit = ?,
			  classify_id = ?,
			  move_type=?,
			  move_frequency=?,
			  calculate_formula=?,
			  modify_time = NOW()
			WHERE edb_info_id = ? `
	_, err = to.Raw(sql, req.EdbName, req.EdbName, req.Frequency, req.Unit, req.ClassifyId, req.MoveType, req.MoveFrequency, req.Formula, edbInfoId).Exec()
	if err != nil {
		return
	}

	var existCondition string
	var existPars []interface{}
	existCondition += " AND edb_info_id=? "
	existPars = append(existPars, edbInfoId)

	existCondition += " AND from_edb_info_id=? "
	existPars = append(existPars, req.FromEdbInfoId)

	//判断计算指标是否被更换
	count, err := GetEdbInfoCalculateCountByCondition(req.Source, existCondition, existPars)
	if err != nil {
		err = errors.New("判断指标是否改变失败,Err:" + err.Error())
		return
	}
	if count <= 0 || req.MoveType != oldEdbInfo.MoveType || req.MoveFrequency != oldEdbInfo.MoveFrequency || req.Formula != oldEdbInfo.CalculateFormula {
		if count <= 0 {
			//删除，计算指标关联的，基础指标的关联关系
			sql = ` DELETE FROM edb_info_calculate_mapping WHERE edb_info_id = ? `
			_, err = to.Raw(sql, edbInfoId).Exec()
			if err != nil {
				err = errors.New("删除计算指标关联关系失败,Err:" + err.Error())
				return
			}
			//关联关系
			{
				calculateMappingItem := new(EdbInfoCalculateMapping)
				calculateMappingItem.CreateTime = time.Now()
				calculateMappingItem.ModifyTime = time.Now()
				calculateMappingItem.Sort = 1
				calculateMappingItem.EdbCode = edbCode
				calculateMappingItem.EdbInfoId = edbInfoId
				calculateMappingItem.FromEdbInfoId = fromEdbInfo.EdbInfoId
				calculateMappingItem.FromEdbCode = fromEdbInfo.EdbCode
				calculateMappingItem.FromEdbName = fromEdbInfo.EdbName
				calculateMappingItem.FromSource = fromEdbInfo.Source
				calculateMappingItem.FromSourceName = fromEdbInfo.SourceName
				calculateMappingItem.FromTag = ""
				calculateMappingItem.Source = utils.DATA_SOURCE_CALCULATE_TIME_SHIFT
				calculateMappingItem.SourceName = "时间移位"
				_, err = to.Insert(calculateMappingItem)
				if err != nil {
					return
				}
			}
		}
		//清空原有数据
		sql = ` DELETE FROM edb_data_calculate_time_shift WHERE edb_info_id = ? `
		_, err = to.Raw(sql, edbInfoId).Exec()
		if err != nil {
			return edbInfoId, err
		}

		edbInfoIdStr := strconv.Itoa(edbInfoId)

		var shiftDay int
		formulaInt, _ := strconv.Atoi(req.Formula)
		switch req.MoveFrequency {
		case "天":
			shiftDay = formulaInt
		case "周":
			shiftDay = formulaInt * 7
		case "月":
			shiftDay = formulaInt * 30
		case "季":
			shiftDay = formulaInt * 90
		case "年":
			shiftDay = formulaInt * 365
		default:
			shiftDay = formulaInt
		}

		//计算数据
		var condition string
		var pars []interface{}
		condition += " AND edb_info_id=? "
		pars = append(pars, req.FromEdbInfoId)
		fmt.Println("EdbInfoId:", req.FromEdbInfoId)
		dataList, err := GetEdbDataListAll(condition, pars, fromEdbInfo.Source, fromEdbInfo.SubSource, 0)
		if err != nil {
			return edbInfoId, err
		}
		addSql := ` INSERT INTO edb_data_calculate_time_shift(edb_info_id,edb_code,data_time,value,create_time,modify_time,status,data_timestamp) values `
		var isAdd bool
		existMap := make(map[string]string)
		dataLen := len(dataList)
		if req.MoveType == 2 {
			shiftDay = -shiftDay
		}
		for i := 0; i < dataLen; i++ {
			//当期
			currentItem := dataList[i]
			currentDate, _ := time.Parse(utils.FormatDate, currentItem.DataTime)
			newDate := currentDate.AddDate(0, 0, shiftDay)
			existKey := edbCode + newDate.Format(utils.FormatDate)
			if _, ok := existMap[existKey]; !ok {
				timestamp := newDate.UnixNano() / 1e6
				timestampStr := fmt.Sprintf("%d", timestamp)
				valStr := decimal.NewFromFloat(currentItem.Value).String()
				addSql += GetAddSql(edbInfoIdStr, edbCode, newDate.Format(utils.FormatDate), timestampStr, valStr)
				isAdd = true
			}
			existMap[existKey] = currentItem.DataTime
		}
		if isAdd {
			addSql = strings.TrimRight(addSql, ",")
			_, err = to.Raw(addSql).Exec()
			if err != nil {
				return edbInfoId, err
			}
		}
	}
	return
}
